/*
 * QEMU low level functions
 *
 * Copyright (c) 2003 Fabrice Bellard
 * Copyright (c) 2006 Dr. Juergen Pfennig (info@j-pfennig.de)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#ifndef QEMU_OSDEP_H
#define QEMU_OSDEP_H

// avoid multiple inclusions if called from vl.h
#ifndef VL_H
    // for printf(...)
    #include <stdarg.h>
#endif

/* specific kludges for OS compatibility (should be moved elsewhere) */
#if defined(__i386__) && !defined(CONFIG_SOFTMMU) && !defined(CONFIG_USER_ONLY)

    /* disabled pthread version of longjmp which prevents us from using an
    alternative signal stack */
    extern void __longjmp(jmp_buf env, int val);
    #define longjmp __longjmp

    #include <signal.h>

    /* NOTE: it works only because the glibc sigset_t is >= kernel sigset_t */
    struct qemu_sigaction {
        union {
            void (*_sa_handler)(int);
            void (*_sa_sigaction)(int, struct siginfo *, void *);
        } _u;
        unsigned long sa_flags;
        void (*sa_restorer)(void);
        sigset_t sa_mask;       /* mask last for extensibility */
    };

    int qemu_sigaction(int signum, const struct qemu_sigaction *act,
                    struct qemu_sigaction *oldact);

    #undef sigaction
    #undef sa_handler
    #undef sa_sigaction
    #define sigaction qemu_sigaction
    #define sa_handler  _u._sa_handler
    #define sa_sigaction    _u._sa_sigaction

    #define     QEMU_EXPLICIT_MALLOC

#endif

#ifndef QEMU_EXPLICIT_MALLOC
    #include <stdlib.h>
    #include <string.h>

    #define     qemu_malloc(a)      malloc(a)
    #define     qemu_mallocz(a)     calloc((a),1)
    #define     qemu_free(a)        free(a)
    #define     qemu_strdup(a)      strdup(a)
    #define     get_mmap_addr(s)    ((void*)0)
#else
    void *qemu_malloc(size_t size);
    void *qemu_mallocz(size_t size);
    void  qemu_free(void *ptr);
    char *qemu_strdup(const char *str);

    void *get_mmap_addr(unsigned long size);
#endif

// resize() is not portable enough? This one should be ...
void *qemu_mresize(void *oldptr, size_t oldsize, size_t newsize);

void *qemu_vmalloc(size_t size);
void  qemu_vfree(void *ptr);

int  qemu_vsnprintf(char *buf, int buflen, const char *fmt, va_list args);
void qemu_vprintf(const char *fmt, va_list ap);
void qemu_printf(const char *fmt, ...);

#endif
